/* CachedMap.cpp
 * Author: Jeroen van Montfort
 * 
 * +------------Oo.------+
 * | Transport Empire -> |
 * +---------------------+
 * 
 * 
 */

#include "CachedMap.h"
#include "MapRequestPacket.h"
#include <iostream>
using namespace std;
using TEmpire::CachedMap;

// Static variables
std::string CachedMap::Author = "Hellfire";
std::string CachedMap::Name = "CachedMap";
std::string CachedMap::Abbreviation = "CMAP";
long CachedMap::Count = 0;

// Constructor
CachedMap::CachedMap() {
  Count++;
}

// Destructor
CachedMap::~CachedMap() {
  Count--;
}

void CachedMap::SetClient(ClientSocket *Client) {
  this->Client = Client;
}

// Create an empty map. This is a map without Tile objects.
void CachedMap::CreateEmptyMap(unsigned short width, unsigned short height) {
  if(Tiles != NULL) {
    if(0 < Width && 0 < Height) {
      for(int i = 0; i < Width * Height; i++) {
        if(Tiles[i] != NULL) {
          delete Tiles[i];
        }
      }
    }
    
    delete Tiles;
  }
  
  Width = width;
  Height = height;
  Tiles = new Tile*[Width * Height];
  for(int i = 0; i < Width * Height; i++) {
    Tiles[i] = NULL;
  }
}

// Prefetch a piece of the map from the server
void CachedMap::Prefetch(unsigned short xStart, unsigned short yStart, 
      unsigned short xEnd, unsigned short yEnd) {
  // TODO: Check whether the region is within the bounds of the map.
  
  // Check if we already have all the tiles
  bool haveAll = true;
  for(unsigned short y = yStart; y <= yEnd; y++) {
    for(unsigned short x = xStart; x <= xEnd; x++) {
      haveAll = haveAll && (GetTile(x, y) != NULL);
    }
  }
  if(haveAll) {
    cout << "We already have this piece of the map!" << endl;
    return;
  }
  
  cout << "requesting..." << endl;
  
  // Send a request to the server
  MapRequestPacket mrp;
  mrp.XStart = xStart;
  mrp.XEnd = xEnd;
  mrp.YStart = yStart;
  mrp.YEnd = yEnd;
  
  Client->SendPacket(&mrp);
}

// Process received tiles
void CachedMap::Process(MapChunkPacket *packet) {
  unsigned short x, y;
  
  unsigned int i = 0;
  for(y = packet->YStart; y <= packet->YEnd; y++) {
    for(x = packet->XStart; x <= packet->XEnd; x++) {
      if(packet->ReceivedTiles[i] == NULL) {
        i++;
      } else {
        if(Tiles[Width * y + x] == NULL) {
          // No tile yet, MOVE the tile in the packet to the map.
          Tiles[Width * y + x] = packet->ReceivedTiles[i];
          packet->ReceivedTiles[i++] = NULL;
        } else {
          // Update existing tile
          Tiles[Width * y + x]->Height = packet->ReceivedTiles[i++]->Height;
        }
      }
    }
  }
}

