/* ClientSocket.cpp
 * Author: Jeroen van Montfort
 * 
 * +------------Oo.------+
 * | Transport Empire -> |
 * +---------------------+
 * 
 * 
 */

#include "ClientSocket.h"
#include "PacketFactory.h"

#include "RakNetworkFactory.h"

using TEmpire::ClientSocket;

// Static variables
std::string ClientSocket::Author = "Hellfire";
std::string ClientSocket::Name = "ClientSocket";
std::string ClientSocket::Abbreviation = "CLSC";
long ClientSocket::Count = 0;


// Constructor
ClientSocket::ClientSocket() {
  rakClient = RakNetworkFactory::GetRakClientInterface();
  
  Count++;
}

// Destructor
ClientSocket::~ClientSocket() {
  Count--;
  
  RakNetworkFactory::DestroyRakClientInterface(rakClient);
}

// Connect to a server
bool ClientSocket::Connect(std::string hostname, unsigned short port) {
  return rakClient->Connect(hostname.c_str(), port, 0, 0, 30);
}

// Is this class connected to a server?
bool ClientSocket::IsConnected() {
  return rakClient->IsConnected();
}

// Send a packet
bool ClientSocket::SendPacket(TEmpire::Packet *packet) {
  unsigned char *buffer;
  unsigned int length = packet->Length();
  
  buffer = new unsigned char[length];
  
  packet->FillBuffer(buffer, length);
  
  return rakClient->Send((char*)buffer, length, HIGH_PRIORITY, RELIABLE_ORDERED, 0);
}

// Receive a packet
TEmpire::Packet* ClientSocket::ReceivePacket() {
  ::Packet *packet = rakClient->Receive();
  
  if(packet == NULL) {
    return NULL;
  }
  
  PacketFactory *factory = PacketFactory::GetInstance();
  TEmpire::Packet *tePacket = 
    factory->Reconstruct(packet->data, packet->length);
  
  rakClient->DeallocatePacket(packet);
  
  return tePacket;
}
