#include "ClientSocket.h"
#include "MapRequestPacket.h"
#include "MapChunkPacket.h"
#include "PacketTypeId.h"
#include "CachedMap.h"

#include <iostream>

#ifdef WIN32
  #include <windows.h>
  #define snooze(n) Sleep(n)
#else
  #include <unistd.h>
  #define snooze(n) usleep(n * 1000);
#endif

using namespace std;
using TEmpire::ClientSocket;
using TEmpire::CachedMap;

void printmap(CachedMap *map) {
  for(int y = 0; y < 16; y++) {
    for(int x = 0; x < 16; x++) {
      Tile* tile = map->GetTile(x, y);
      if(tile == NULL) {
        printf("xx ");
      } else {
        printf("%02d ", tile->Height);
      }
    }
    printf("\n");
  }
}

int main(int argc, char *argv[]) {

  // Create connection
  ClientSocket client;
  if(!client.Connect("localhost", 4000)) {
    return -1;
  }
  
  // Create map
  CachedMap map;
  map.CreateEmptyMap(16, 16);
  map.SetClient(&client);
  
  cout << "Connecting..." << endl;
  while(!client.IsConnected()) {
    snooze(30);
  }
  cout << "Connected!" << endl;
  
  // Request tiles (0, 0) through (7, 7)
  cout << "Requesting (0, 0) - (7, 7)" << endl;
  map.Prefetch(0, 0, 7, 7);
    
  // Receive packets
  int iteration = 0;
  while(true) {
    // Do something funny at the 100th and 200th iteration
    if(iteration == 100) {
      // Request (8, 8) through (15, 15)
      cout << "Requesting (8, 8) - (15, 15)" << endl;
      map.Prefetch(8, 8, 15, 15);
    } else if(iteration == 200) {
      // Request (0, 0) through (7, 7) again. This should not lead to communication
      cout << "Requesting (0, 0) - (7, 7)" << endl;
      map.Prefetch(0, 0, 7, 7);
    } else if(iteration == 300) {
      cout << "Our map now looks like this:" << endl;
      printmap(&map);
    }
    iteration++;
    
    TEmpire::Packet *packet = client.ReceivePacket();
    
    if(packet == NULL) {
      snooze(30);
      continue;
    }
  
    cout << "Packet received!" << endl;
    
    switch(packet->GetType()) {
      case TEmpire::PCID_MAPCHUNK:
        cout << "Got MapChunkPacket. Processing..." << endl;
        map.Process((TEmpire::MapChunkPacket*)packet);
        break;
    }
    
    delete packet;
  }
  
  return 0;
}
