/* MapChunkPacket.cpp
 * Author: Jeroen van Montfort
 * 
 * +------------Oo.------+
 * | Transport Empire -> |
 * +---------------------+
 * 
 * 
 */

#include "MapChunkPacket.h"
#include "PacketTypeId.h"
#include <stdio.h>

using TEmpire::MapChunkPacket;

// Static variables
std::string MapChunkPacket::Author = "Hellfire";
std::string MapChunkPacket::Name = "MapChunkPacket";
std::string MapChunkPacket::Abbreviation = "MPCP";
long MapChunkPacket::Count = 0;

// Constructor
MapChunkPacket::MapChunkPacket() {
  Count++;
}

// Destructor
MapChunkPacket::~MapChunkPacket() {
  Count--;
  
  if(ReceivedTiles != NULL) {
    for(int i = 0; i < (XEnd - XStart + 1) * (YEnd - YStart + 1); i++) {
      if(ReceivedTiles[i] != NULL) {
        delete ReceivedTiles[i];
      }
    }
    delete ReceivedTiles;
  }
}

// Return the packet's type identifier
unsigned char MapChunkPacket::GetType() {
  return PCID_MAPCHUNK;
}

unsigned long MapChunkPacket::Length() {
  return 9 + (2 * (XEnd - XStart + 1) * (YEnd - YStart + 1));
}

unsigned long MapChunkPacket::FillBuffer(unsigned char *buffer, unsigned long bufLen) {
  // All packets start with a 8-bit type identifier.
  
  unsigned long length = Length();
  unsigned long offset = 0;
  
  // Don't allow buffers that are too small
  if(bufLen < length) {
    return 0;
  }
  
  // Add type  to the buffer
  buffer[offset++] = GetType();
  
  // Add X and Y range to the buffer
  offset = AddUShort(XStart, buffer, offset);
  offset = AddUShort(XEnd, buffer, offset);
  offset = AddUShort(YStart, buffer, offset);
  offset = AddUShort(YEnd, buffer, offset);
  
  // Add individual tiles to the packet
  for(int y = YStart; y <= YEnd; y++) {
    for(int x = XStart; x <= XEnd; x++) {
      offset = AddUShort(PMap->GetTile(x, y)->Height, buffer, offset);
    }
  }
  
  return offset;
}

// Reconstruct ourselfs from a serialized version
bool MapChunkPacket::ParseBuffer(unsigned char *buffer, unsigned long bufLen) {
  unsigned char type;
  unsigned int offset = 0;
  
  // Packets have a minimum length of 4
  if(bufLen < 4) {
    return false;
  }
  
  type = buffer[offset++];
 
  // Parse X and Y range
  offset = ParseUShort(XStart, buffer, offset);
  offset = ParseUShort(XEnd, buffer, offset);
  offset = ParseUShort(YStart, buffer, offset);
  offset = ParseUShort(YEnd, buffer, offset);
  
  // Parse individual tiles
  unsigned short width = XEnd - XStart + 1;
  ReceivedTiles = new Tile*[width * (YEnd - YStart + 1)];

  // Fill individual tiles
  for(unsigned short y = 0; y < YEnd - YStart + 1; y++) {
    for(unsigned short x = 0; x < XEnd - XStart + 1; x++) {
      Tile* tile = new Tile();
      offset = ParseUShort(tile->Height, buffer, offset);

      ReceivedTiles[width * y + x] = tile;
    }
  }
  
  return true;
}
