/* MapRequestPacket.cpp
 * Author: Jeroen van Montfort
 * 
 * +------------Oo.------+
 * | Transport Empire -> |
 * +---------------------+
 * 
 * 
 */

#include "MapRequestPacket.h"
#include "PacketTypeId.h"

using TEmpire::MapRequestPacket;

// Static variables
std::string MapRequestPacket::Author = "Hellfire";
std::string MapRequestPacket::Name = "MapRequestPacket";
std::string MapRequestPacket::Abbreviation = "MPRP";
long MapRequestPacket::Count = 0;

// Constructor
MapRequestPacket::MapRequestPacket() {
  XStart = XEnd = YStart = YEnd = 0;
  
  Count++;
}

// Destructor
MapRequestPacket::~MapRequestPacket() {
  Count--;
}

// Return the packet's type identifier
unsigned char MapRequestPacket::GetType() {
  return PCID_MAPREQUEST;
}

unsigned long MapRequestPacket::Length() {
  return 9;
}

unsigned long MapRequestPacket::FillBuffer(unsigned char *buffer, unsigned long bufLen) {
  unsigned long length = Length();
  unsigned long offset = 0;
  
  // Don't allow buffers that are too small
  if(bufLen < length) {
    return 0;
  }
  
  // Add type to the buffer
  buffer[offset++] = GetType();
  
  // Add X and Y range to the buffer
  offset = AddUShort(XStart, buffer, offset);
  offset = AddUShort(XEnd, buffer, offset);
  offset = AddUShort(YStart, buffer, offset);
  offset = AddUShort(YEnd, buffer, offset);
  
  return offset;
}

// Reconstruct ourselfs from a serialized version
bool MapRequestPacket::ParseBuffer(unsigned char *buffer, unsigned long bufLen) {
  unsigned char type;
  unsigned int offset = 0;
  
  if(bufLen < Length()) {
    return false;
  }
  
  // Skip type
  offset++;
  
  // Parse X and Y range
  offset = ParseUShort(XStart, buffer, offset);
  offset = ParseUShort(XEnd, buffer, offset);
  offset = ParseUShort(YStart, buffer, offset);
  offset = ParseUShort(YEnd, buffer, offset);
  
  return true;
}
