#ifndef __PACKET_H__
#define __PACKET_H__

/* Packet.h
 * Author: Jeroen van Montfort
 * 
 * +------------Oo.------+
 * | TRANSPORT EMPIRE -> |
 * +---------------------+
 * 
 * 
 */

namespace TEmpire {

/* Class Packet:
 * 
 * Provides a generic interface to small classes that can be serialized for
 * transmission over a network.
 */

class Packet {
public:
  // Return the packet's type identifier
  virtual unsigned char GetType() = 0;
  
  // Calculate the number of bytes this packet will use
  virtual unsigned long Length() = 0;
  
  // Fill a buffer with the data in the packet
  virtual unsigned long FillBuffer(unsigned char *buffer, 
    unsigned long buflen) = 0;
  
  // Reconstruct ourselfs from a serialized version
  virtual bool ParseBuffer(unsigned char *buffer, unsigned long buflen) = 0;

protected:
  // Helper methods
  
  // Add an unsigned short to a buffer
  unsigned int AddUShort(unsigned short value, unsigned char *buffer, int offset = 0);
  
  // Parse an unsigned short from a buffer
  unsigned int ParseUShort(unsigned short &value, unsigned char *buffer, int offset = 0);
};

} // namespace TEmpire

#endif // __PACKET_H__
