/* PacketFactory.cpp
 * Author: Jeroen van Montfort
 * 
 * +------------Oo.------+
 * | TRANSPORT EMPIRE -> |
 * +---------------------+
 * 
 * 
 */

#include "PacketFactory.h"
#include "PacketTypeId.h"
#include "MapRequestPacket.h"
#include "MapChunkPacket.h"

#include <iostream>

using namespace TEmpire;
using std::cerr;
using std::endl;

// Static variables
PacketFactory *PacketFactory::Instance = NULL;

std::string PacketFactory::Author = "Hellfire";
std::string PacketFactory::Name = "PacketFactory";
std::string PacketFactory::Abbreviation = "PFCT";
long PacketFactory::Count = 0;

// Constructor
PacketFactory::PacketFactory() {
}

// Get the only object of the PacketFactory class
PacketFactory *PacketFactory::GetInstance() {
  if(Instance == NULL) {
    Instance = new PacketFactory();
  }
  
  return Instance;
}

// Reconstruct a packet from a stream buffer
Packet *PacketFactory::Reconstruct(unsigned char *buffer, unsigned int bufLen) {
  unsigned char type;
  
  type = buffer[0];
  
  Packet *packet = NULL;
  
  switch(type) {
    case PCID_MAPREQUEST:
      packet = new MapRequestPacket();
      break;
    case PCID_MAPCHUNK:
      packet = new MapChunkPacket();
      break;
    
    default:
      cerr << "PacketFactory: I can't handle a packet of type " << (int)type <<
        " (yet)!" << endl;
      return NULL;
  }
  
  packet->ParseBuffer(buffer, bufLen);
  
  return packet;
}
