/* Socket.cpp
 * Author: Jeroen van Montfort
 * 
 * +------------Oo.------+
 * | TRANSPORT EMPIRE -> |
 * +---------------------+
 * 
 * 
 */

#include "Socket.h"
#include "PacketFactory.h"
#include <iostream>

#ifdef WIN32
  #include <winsock2.h>
#else
  #include <unistd.h>
  #include <sys/socket.h>
  #include <arpa/inet.h>
#endif

using namespace std;
using TEmpire::Socket;

// Static variables
std::string Socket::Author = "Hellfire";
std::string Socket::Name = "Socket";
std::string Socket::Abbreviation = "SOCK";
long Socket::Count = 0;

Socket::Socket() {
  // Initialize Winsock2 on Windows platforms
  #ifdef WIN32
  if(Count == 0) {
    WSADATA wsaData;
    int err = WSAStartup(0x0002, &wsaData);
    if(err) {
      cerr << "Error starting Winsock2" << endl;
    }
  }
  #endif

  Sock = 0;
  Count++;
}

Socket::~Socket() {
  Count--;
  
  if(Sock != 0) {
    #ifdef WIN32
      closesocket(Sock);
    #else
      close(Sock);
    #endif
  }
  
  // Deinitialize Winsock2 on Windows platforms
  #ifdef WIN32
  if(Count == 0) {
    WSACleanup();
  }
  #endif
}

// Send a packet over the socket
bool Socket::SendPacket(Packet *packet) {
  unsigned char *buffer;
  unsigned long bufLen = packet->Length();
  
  buffer = new unsigned char[bufLen];
  packet->FillBuffer(buffer, bufLen);
  
  send(Sock, (char*)buffer, bufLen, 0);
  
  delete buffer;
  
  return true;
}

// Receive a packet
TEmpire::Packet* Socket::ReceivePacket() {
  unsigned char *buffer = new unsigned char[4];
  
  int bytes = recv(Sock, (char*)buffer, 4, MSG_PEEK);
  
  if(bytes != 4) {
    delete buffer;
    return NULL;
  }
  
  unsigned short type = buffer[0] << 8 | buffer[1];
  unsigned short length = buffer[2] << 8 | buffer[3];
  
  if(4 < length) {
    delete buffer;
    buffer = new unsigned char[length];
    bytes = recv(Sock, (char*)buffer, length, 0);
  }
  
  Packet *packet = PacketFactory::GetInstance()->Reconstruct(buffer, length);
  
  // Clean up
  delete buffer;
  
  return packet;
}
