/* ClientSocket.cpp
 * Author: Jeroen van Montfort
 * 
 * +------------Oo.------+
 * | Transport Empire -> |
 * +---------------------+
 * 
 * 
 */

#include "ServerSocket.h"
#include "PacketFactory.h"

#include "RakNetworkFactory.h"

using TEmpire::ServerSocket;

// Static variables
std::string ServerSocket::Author = "Hellfire";
std::string ServerSocket::Name = "ClientSocket";
std::string ServerSocket::Abbreviation = "CLSC";
long ServerSocket::Count = 0;


// Constructor
ServerSocket::ServerSocket() {
  rakServer = RakNetworkFactory::GetRakServerInterface();
  
  Count++;
}

// Destructor
ServerSocket::~ServerSocket() {
  Count--;
  
  RakNetworkFactory::DestroyRakServerInterface(rakServer);
}

// Open a port
bool ServerSocket::OpenPort(unsigned short port) {
  // Start a server for a maximum of 16 players. This value might change in the
  // future. For the current task (task 3 at the moment of writing), 16 will be
  // more than sufficient.
  // TODO: Set to a sensible limit.
  return rakServer->Start(16, 0, 30, port);
}

// Send a packet
bool ServerSocket::SendPacket(TEmpire::Packet *packet) {
  unsigned char *buffer;
  unsigned int length = packet->Length();
  
  buffer = new unsigned char[length];
  
  packet->FillBuffer(buffer, length);
  
  // Broadcast the packet to all players. TODO: add some player selection
  // mechanism.
  return rakServer->Send((char*)buffer, length, HIGH_PRIORITY,
    RELIABLE_ORDERED, 0, UNASSIGNED_PLAYER_ID, true);
}

// Receive a packet
TEmpire::Packet* ServerSocket::ReceivePacket() {
  ::Packet *packet = rakServer->Receive();
  
  if(packet == NULL) {
    return NULL;
  }
  
  PacketFactory *factory = PacketFactory::GetInstance();
  TEmpire::Packet *tePacket = 
    factory->Reconstruct(packet->data, packet->length);
  
  rakServer->DeallocatePacket(packet);
  
  return tePacket;
}
