#include "ServerSocket.h"
#include "PacketTypeId.h"
#include "MapRequestPacket.h"
#include "MapChunkPacket.h"
#include "Map.h"


#include <iostream>

#ifdef WIN32
  #include <windows.h>
  #define snooze(n) Sleep(n)
#else
  #include <unistd.h>
  #define snooze(n) usleep(n * 1000);
#endif

using namespace std;
using TEmpire::ServerSocket;
using TEmpire::Map;

int main(int argc, char *argv[]) {
  
  // Create a random map
  Map *map = new Map();
  map->Random(16, 16);
  
  printf("The map looks like this:\n");
  for(unsigned short y = 0; y < 16; y++) {
    for(unsigned short x = 0; x < 16; x++) {
      printf("%02d ", map->GetTile(x, y)->Height);
    }
    printf("\n");
  }
  
  // Create connection
  ServerSocket *server = new ServerSocket();
  if(!server->OpenPort(4000)) {
    delete server;
    
    return -1;
  }
  
  snooze(30);
  
  // Receive packets
  while(true) {
    TEmpire::Packet *packet = server->ReceivePacket();
    
    if(packet == NULL) {
      snooze(30);
      continue;
    }
  
    cout << "Packet received!" << endl;

    switch(packet->GetType()) {
      case TEmpire::PCID_MAPREQUEST:
        TEmpire::MapRequestPacket *mrp = (TEmpire::MapRequestPacket*)packet;
        
        cout << "MapRequestPacket from (" << mrp->XStart << ", " << mrp->YStart << ") to (" << mrp->XEnd << ", " << mrp->YEnd << ")" << endl;
        
        TEmpire::MapChunkPacket mcp;
        mcp.XStart = mrp->XStart;
        mcp.XEnd = mrp->XEnd;
        mcp.YStart = mrp->YStart;
        mcp.YEnd = mrp->YEnd;
        mcp.PMap = map;
        server->SendPacket(&mcp);
        
        cout << "Sent reply." << endl;
        
        break;
    }    

    delete packet;
  }
  
  
  // Clean up
  delete server;
  delete map;
  
  return 0;
}
